__author__ = 'saeedamen'

#
# Copyright 2020 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and limitations under the License.
#

"""
Shows how to use QuickChart to quickly download data and plot it
"""

# Choose run_example = 0 for everything
# run_example = 1 - Plot S&P 500 charts with Matplotlib
# run_example = 1 - Plot FX chart with Plotly

run_example = 0

###### Plot with Matplotlib
if run_example == 1 or run_example == 0:
    from finmarketpy.economics import QuickChart

    # Plot with Matplotlib - S&P 500 on LHS y-axis and S&P 500 net long spec positioning on RHS y-axis
    QuickChart(engine='matplotlib', data_source='bloomberg').plot_chart(tickers={'S&P500' : 'SPX Index'},
        tickers_rhs={'Net long spec S&P 500 futures' : 'IMM0ENCN Index'},
        title='S&P500 vs. net spec pos RHS (2008-2010)',
                start_date='01 Jan 2007', finish_date='01 Jan 2010', source='Bloomberg')

    # Plot with Matplotlib - S&P 500 YoY returns
    QuickChart(engine='matplotlib', data_source='bloomberg').plot_chart(tickers={'S&P500' : 'SPX Index'},
                title='S&P 500 YoY', chart_type='bar',
                start_date='01 Jan 2007', yoy=True, source='Bloomberg')

###### Plot with Plotly and Matplotlib
if run_example == 2 or run_example == 0:
    from finmarketpy.economics import QuickChart

    # Plot with matplotlib - Major USD crosses reindexed from 100 in 2020
    QuickChart(engine='matplotlib', data_source='bloomberg').plot_chart(
        tickers=['EURUSD Curncy', 'GBPUSD Curncy', 'AUDUSD Curncy'],
        title='USD crosses in 2020',
        start_date='01 Jan 2020', reindex=True, source='Bloomberg')

    # Plot with Plotly - Major USD crosses reindexed from 100 in 2020
    QuickChart(engine='plotly', data_source='bloomberg').plot_chart(
        tickers=['EURUSD Curncy', 'GBPUSD Curncy', 'AUDUSD Curncy'],
        title='USD crosses in 2020 (Plotly)',
        start_date='01 Jan 2020', reindex=True, source='Bloomberg')


import numpy as np
import pandas as pd
from finmarketpy.util.marketconstants import MarketConstants

from findatapy.util import SwimPool
from findatapy.util import LoggerManager
from finmarketpy.backtest.backtestengine import TradingModel
from chartpy import Chart, Style, ChartConstants


class BacktestComparison(object):

    DEFAULT_PLOT_ENGINE = ChartConstants().chartfactory_default_engine
    SCALE_FACTOR = ChartConstants().chartfactory_scale_factor
    CHART_SOURCE = ChartConstants().chartfactory_source
    CHART_STYLE = Style()

    def __init__(self, models, ref_index=0,
                 labels=None):
        """

        :param models: iterable of TradingModel instances.
        :param ref_index: index of the reference model in the list (for difference).
        """
        if hasattr(models, "__iter__") and all([isinstance(x, TradingModel) for x in models]):
            self.models = models
            self.ref_index = ref_index
        else:
            raise AttributeError("Models need to be an iterable of TradingModel instances.")

        self.labels = labels

    def plot_pnl(self, diff=True, silent_plot=False, reduce_plot=True):
        style = self.models[self.ref_index]._create_style("", "Strategy PnL", reduce_plot=reduce_plot)

        models = self.models
        ref = self.ref_index

        pnls = [model._strategy_pnl for model in models]

        df = pd.concat(pnls, axis=1)

        if diff:
            df = df.subtract(pnls[ref], axis='index')
        if self.labels is not None:
            df.columns = self.labels

        chart = Chart(df, engine=self.DEFAULT_PLOT_ENGINE, chart_type='line', style=style)
        if not silent_plot:
            chart.plot()

        return chart

    def plot_sharpe(self, silent_plot=False, reduce_plot=True):
        style = self.models[self.ref_index]._create_style("", "Sharpe Curve", reduce_plot=reduce_plot)

        models = self.models
        ref = self.ref_index

        returns = [model._strategy_pnl.pct_change() for model in models]
        stdev_of_returns = np.std(returns)

        annualized_sharpe = returns / stdev_of_returns * np.sqrt(250)

        df = pd.concat(annualized_sharpe, axis=1)

        chart = Chart(df, engine=self.DEFAULT_PLOT_ENGINE, chart_type='bar', style=style)
        if not silent_plot:
            chart.plot()
        return chart

    def plot_strategy_trade_notional(self, diff=True, silent_plot=False, reduce_plot=True):
        style = self.models[self.ref_index]._create_style("", "Trades (Scaled by Notional)", reduce_plot=reduce_plot)

        models = self.models
        ref = self.ref_index

        strategy_trade_notional = [model._strategy_trade_notional for model in models]

        df = pd.concat(strategy_trade_notional, axis=1)

        if diff:
            df = df.subtract(strategy_trade_notional[ref], axis='index')
        if self.labels is not None:
            df.columns = self.labels

        chart = Chart(df, engine=self.DEFAULT_PLOT_ENGINE, chart_type='bar', style=style)
        if not silent_plot:
            chart.plot()

        return chart



__author__ = 'saeedamen'

#
# Copyright 2020 Cuemacro
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and limitations under the License.
#

"""
Shows how to download market data from Quandl
"""


# loading data
import datetime

from chartpy import Chart, Style
from findatapy.market import Market, MarketDataGenerator, MarketDataRequest

from findatapy.util.loggermanager import LoggerManager

logger = LoggerManager().getLogger(__name__)

chart = Chart(engine='matplotlib')

market = Market(market_data_generator=MarketDataGenerator())

# Choose run_example = 0 for everything
# run_example = 1 - download

run_example = 0

###### fetch data from Quandl for BoE rate (using Bloomberg data)
if run_example == 1 or run_example == 0:
    # Monthly average of UK resident monetary financial institutions' (excl. Central Bank) sterling
    # Weighted average interest rate, other loans, new advances, on a fixed rate to private non-financial corporations (in percent)
    # not seasonally adjusted
    md_request = MarketDataRequest(
        start_date="01 Jan 2000",  # start date
        data_source='quandl',  # use Quandl as data source
        tickers=['Weighted interest rate'],
        fields=['close'],  # which fields to download
        vendor_tickers=['BOE/CFMBJ84'],  # ticker (Bloomberg)
        vendor_fields=['close'],  # which Bloomberg fields to download
        cache_algo='internet_load_return',
        quandl_api_key='mchs47VHfnWzxv6mGhxA')  # how to return data

    df = market.fetch_market(md_request)

    style = Style()

    style.title = 'BoE weighted interest rate'
    style.scale_factor = 3
    style.file_output = "boe-rate.png"
    style.source = 'Quandl/BoE'

    chart.plot(df, style=style)